# Tcl package index file, version 1.1
# This file is generated by the "pkg_mkIndex" command
# and sourced either when an application starts up or
# by a "package unknown" script.  It invokes the
# "package ifneeded" command to set up package-related
# information so that packages will be loaded automatically
# in response to "package require" commands.  When this
# script is sourced, the variable $dir must contain the
# full path name of this file's directory.

package ifneeded tclAE 2.0 [list load [file join $dir Contents MacOS tclAE2.0.dylib]]\n[ \
list tclPkgSetup [file join $dir Contents Resources] tclAE 2.0 { \
	{aebuild.tcl source {::tclAE::build::throw ::tclAE::build::event ::tclAE::build::resultDataAs ::tclAE::build::resultData ::tclAE::build::resultDescAs ::tclAE::build::resultDesc ::tclAE::build::protect ::tclAE::build::objectProperty ::tclAE::build::coercion ::tclAE::build::List ::tclAE::build::hexd ::tclAE::build::_ensureBinary ::tclAE::build::bool ::tclAE::build::TEXT ::tclAE::build::alis ::tclAE::build::fss ::tclAE::build::path ::tclAE::build::enum ::tclAE::build::name ::tclAE::build::filename ::tclAE::build::winByName ::tclAE::build::winByPos ::tclAE::build::lineRange ::tclAE::build::charRange ::tclAE::build::absPos ::tclAE::build::nullObject ::tclAE::build::objectType ::tclAE::build::nameObject ::tclAE::build::indexObject ::tclAE::build::everyObject ::tclAE::build::rangeObject ::tclAE::build::propertyObject ::tclAE::build::propertyListObject ::tclAE::build::startupDisk ::tclAE::build::userName ::tclAE::build::foldername}} \
	{aelegacy.tcl source {::tclAE::build ::tclAE::print ::tclAE::disposeDesc ::tclAE::encode ::tclAE::legacyQueueHandler}} \
	{aete.tcl source {::tclAE::handleGetAETE ::tclAE::aete::register ::tclAE::aete::deregister ::tclAE::buildAETE ::tclAE::aete::build::buildAETE ::tclAE::aete::build::buildSuite ::tclAE::aete::build::buildEvent ::tclAE::aete::build::buildParameter ::tclAE::aete::build::buildEnumeration ::tclAE::aete::build::buildEnumerator ::tclAE::aete::build::buildList ::tclAE::aete::build::buildDirectParameter ::tclAE::aete::build::buildReplyParameter ::tclAE::aete::build::buildString ::tclAE::aete::build::buildIdentification ::tclAE::parseAETE ::tclAE::aete::parse::parseAETE ::tclAE::aete::parse::parseSuite ::tclAE::aete::parse::parseEvent ::tclAE::aete::parse::parseClass ::tclAE::aete::parse::parseCompEnum ::tclAE::aete::parse::parseEnumeration ::tclAE::aete::parse::parseIdentification ::tclAE::aete::parse::parseMainParameter ::tclAE::aete::parse::parseParameter ::tclAE::aete::parse::parseString ::tclAE::aete::parse::parseElementClass}} \
	{oserrors.tcl source {::error::throwOSErr}} \
}]
