#ifndef _LINPHONEWRAP
#define _LINPHONEWRAP

#include <tcl.h>
#include <stdio.h>
#include <stdio.h>
#include <stdlib.h>

/*
 * Only the _Init function is exported.
 */

EXTERN int	Linphone_Init _ANSI_ARGS_((Tcl_Interp * interp));

#endif
