#include <applet-widget.h>

#define SESSION_HAND 1
#define ST_ONLINE 15
#define ST_OFFLINE 16
#define ST_NOACT 17
#define ST_BUSY 18
#define ST_BRB 19
#define ST_AWAY 20
#define ST_ONPHONE 21
#define ST_LUNCH 22
#define ST_APP_OFFLINE 23

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <gnome.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <glib.h>

typedef struct _GAmsnApplet
{
  GtkWidget *applet;
  GtkWidget *frame;
  GtkWidget *eventbox;
  GtkWidget *picbuffer;

  GdkPixmap *online_pic;
  GdkPixmap *offline_pic;
  GdkPixmap *busy_pic;
  GdkPixmap *away_pic;
  GdkPixmap *phone_pic;
  GdkPixmap *brb_pic;
  GdkPixmap *appear_pic;
  GdkPixmap *noact_pic;
  GdkPixmap *lunch_pic;
  
  FILE *stream;
  gint status;
  char *path;
}

GAmsnApplet;

